import * as sitem from './StackItem.js';
import { Symbol } from './Symbol.js';
import TexParser from './TexParser.js';
import { MmlNode } from '../../core/MmlTree/MmlNode.js';
export declare namespace ParseMethods {
    function MmlFilterAttribute(parser: TexParser, name: string, value: string): string;
    function ControlSequence(parser: TexParser, c: string): void;
    function csMathchar0mi(parser: TexParser, mchar: Symbol): void;
    function csMathchar0mo(parser: TexParser, mchar: Symbol): void;
    function csMathchar7(parser: TexParser, mchar: Symbol): void;
    function csDelimiter(parser: TexParser, delim: Symbol): void;
    function csUndefined(parser: TexParser, name: string): void;
    function envUndefined(parser: TexParser, env: string): void;
    function Variable(parser: TexParser, c: string): void;
    function Number(parser: TexParser, c: string): void;
    function Open(parser: TexParser, c: string): void;
    function Close(parser: TexParser, c: string): void;
    function Tilde(parser: TexParser, c: string): void;
    function Space(parser: TexParser, c: string): void;
    function Superscript(parser: TexParser, c: string): void;
    function Subscript(parser: TexParser, c: string): void;
    function Prime(parser: TexParser, c: string): void;
    function Comment(parser: TexParser, c: string): void;
    function Hash(parser: TexParser, c: string): void;
    function Other(parser: TexParser, c: string): void;
    function SetFont(parser: TexParser, name: string, font: string): void;
    function SetStyle(parser: TexParser, name: string, texStyle: string, style: string, level: string): void;
    function SetSize(parser: TexParser, name: string, size: string): void;
    function Color(parser: TexParser, name: string): void;
    function Spacer(parser: TexParser, name: string, space: string): void;
    function LeftRight(parser: TexParser, name: string): void;
    function Middle(parser: TexParser, name: string): void;
    function NamedFn(parser: TexParser, name: string, id: string): void;
    function NamedOp(parser: TexParser, name: string, id: string): void;
    function Limits(parser: TexParser, name: string, limits: string): void;
    function Over(parser: TexParser, name: string, open: string, close: string): void;
    function Frac(parser: TexParser, name: string): void;
    function Sqrt(parser: TexParser, name: string): void;
    function Root(parser: TexParser, name: string): void;
    function parseRoot(parser: TexParser, n: string): MmlNode;
    function MoveRoot(parser: TexParser, name: string, id: string): void;
    function Accent(parser: TexParser, name: string, accent: string, stretchy: boolean): void;
    function UnderOver(parser: TexParser, name: string, c: string, stack: boolean, noaccent: boolean): void;
    function Overset(parser: TexParser, name: string): void;
    function Underset(parser: TexParser, name: string): void;
    function TeXAtom(parser: TexParser, name: string, mclass: number): void;
    function MmlToken(parser: TexParser, name: string): void;
    function Strut(parser: TexParser, name: string): void;
    function Phantom(parser: TexParser, name: string, v: string, h: string): void;
    function Smash(parser: TexParser, name: string): void;
    function Lap(parser: TexParser, name: string): void;
    function RaiseLower(parser: TexParser, name: string): void;
    function MoveLeftRight(parser: TexParser, name: string): void;
    function Hskip(parser: TexParser, name: string): void;
    function Rule(parser: TexParser, name: string, style: string): void;
    function rule(parser: TexParser, name: string): void;
    function MakeBig(parser: TexParser, name: string, mclass: number, size: number): void;
    function BuildRel(parser: TexParser, name: string): void;
    function HBox(parser: TexParser, name: string, style: string): void;
    function FBox(parser: TexParser, name: string): void;
    function Not(parser: TexParser, name: string): void;
    function Dots(parser: TexParser, name: string): void;
    function Matrix(parser: TexParser, name: string, open: string, close: string, align: string, spacing: string, vspacing: string, style: string, cases: boolean, numbered: boolean): void;
    function Entry(parser: TexParser, name: string): void;
    function Cr(parser: TexParser, name: string): void;
    function CrLaTeX(parser: TexParser, name: string): void;
    function HLine(parser: TexParser, name: string, style: string): void;
    function HFill(parser: TexParser, name: string): void;
    function mi2mo(mi: MmlNode): MmlNode;
    function BeginEnd(parser: TexParser, name: string): void;
    function BeginEnvironment(parser: TexParser, func: Function, env: string, args: any[]): void;
    function Equation(parser: TexParser, begin: string, row: MmlNode[]): MmlNode[];
    function Array(parser: TexParser, begin: sitem.StackItem, open: string, close: string, align: string, spacing: string, vspacing: string, style: string, raggedHeight: boolean): sitem.ArrayItem;
    function AlignedArray(parser: TexParser, begin: sitem.StackItem): sitem.ArrayItem;
    function setArrayAlign(parser: TexParser, array: sitem.ArrayItem, align: string): sitem.ArrayItem;
    function Require(parser: TexParser, name: string): void;
    function Extension(parser: TexParser, name: string | sitem.StackItem, file: string, array?: any): void;
    function Macro(parser: TexParser, name: string, macro: string, argcount: number, def?: string): void;
    function SubstituteArgs(args: string[], string: string): any;
    function AddArgs(s1: string, s2: string): string;
    function AMSarray(parser: TexParser, begin: sitem.StackItem, numbered: boolean, taggable: boolean, align: string, spacing: string): sitem.AMSarrayItem;
    function checkEqnEnv(parser: TexParser): void;
    function HandleOperatorName(parser: TexParser, name: string): void;
    function SkipLimits(parser: TexParser, name: string): void;
    function MathChoice(parser: TexParser, name: string): void;
}
