"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var sm = require("./SymbolMap.js");
var MapHandler_js_1 = require("./MapHandler.js");
var BaseMappings_js_1 = require("./BaseMappings.js");
var AmsMappings_js_1 = require("./AmsMappings.js");
var AmsSymbols_js_1 = require("./AmsSymbols.js");
var ParseMethods_js_1 = require("./ParseMethods.js");
var TreeHelper_js_1 = require("./TreeHelper.js");
var StackItem_js_1 = require("./StackItem.js");
var Stack_js_1 = require("./Stack.js");
var TexError_js_1 = require("./TexError.js");
var TexParser = (function () {
    function TexParser(str, env) {
        this.macroCount = 0;
        this.secondPass = [];
        this.input = '';
        this.remainder = '';
        this.configurations = new Map();
        this.NBSP = '\u00A0';
        this.i = 0;
        this.remap = MapHandler_js_1.default.getInstance().getMap('remap');
        this.configure(BaseMappings_js_1.BaseMappings.CONFIGURATION);
        this.append(AmsMappings_js_1.AmsMappings.CONFIGURATION);
        this.append(AmsSymbols_js_1.AmsSymbols.CONFIGURATION);
        this.setup(ParseMethods_js_1.ParseMethods);
        this.string = str;
        var ENV;
        if (env) {
            ENV = {};
            for (var id in env) {
                if (env.hasOwnProperty(id)) {
                    ENV[id] = env[id];
                }
            }
        }
        this.stack = new Stack_js_1.default(ENV, !!env);
        this.Parse();
        this.Push(new StackItem_js_1.StopItem());
    }
    TexParser.prototype.process = function (tex) { };
    TexParser.prototype.setup = function (env) {
        var maps = MapHandler_js_1.default.getInstance().allMaps();
        for (var i = 0, map = void 0; map = maps[i]; i++) {
            if (map instanceof sm.CharacterMap ||
                map instanceof sm.RegExpMap ||
                map instanceof sm.EnvironmentMap) {
                try {
                    var parser = map.parser(null);
                    if (typeof parser === 'string') {
                        map.parser = env[parser];
                    }
                }
                catch (e) { }
            }
            if (map instanceof sm.MacroMap) {
                map.functionMap = env;
            }
        }
        this.fallback('character', env['Other']);
        this.fallback('macro', env['csUndefined']);
        this.fallback('environment', env['envUndefined']);
    };
    TexParser.prototype.configure = function (config) {
        try {
            for (var _a = __values(Object.keys(config)), _b = _a.next(); !_b.done; _b = _a.next()) {
                var key = _b.value;
                var name_1 = key;
                this.configurations.set(name_1, new SubHandler(config[name_1] || []));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var e_1, _c;
    };
    TexParser.prototype.append = function (config) {
        try {
            for (var _a = __values(Object.keys(config)), _b = _a.next(); !_b.done; _b = _a.next()) {
                var key = _b.value;
                var name_2 = key;
                try {
                    for (var _c = __values(config[name_2]), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var map = _d.value;
                        this.configurations.get(name_2).add(map);
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_d && !_d.done && (_e = _c.return)) _e.call(_c);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_f = _a.return)) _f.call(_a);
            }
            finally { if (e_3) throw e_3.error; }
        }
        var e_3, _f, e_2, _e;
    };
    TexParser.prototype.parse = function (kind, input) {
        return this.configurations.get(kind).parse(input);
    };
    TexParser.prototype.lookup = function (kind, symbol) {
        return this.configurations.get(kind).lookup(symbol);
    };
    TexParser.prototype.fallback = function (kind, method) {
        return this.configurations.get(kind).fallback(method);
    };
    TexParser.prototype.contains = function (kind, symbol) {
        return this.configurations.get(kind).contains(symbol);
    };
    TexParser.prototype.toString = function () {
        var str = '';
        try {
            for (var _a = __values(Array.from(this.configurations.keys())), _b = _a.next(); !_b.done; _b = _a.next()) {
                var config = _b.value;
                str += config + ': ' +
                    this.configurations.get(config) + '\n';
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return str;
        var e_4, _c;
    };
    TexParser.prototype.Parse = function () {
        TreeHelper_js_1.TreeHelper.printMethod('Parse (Old Parser Object)');
        var c, n;
        while (this.i < this.string.length) {
            c = this.string.charAt(this.i++);
            n = c.charCodeAt(0);
            if (n >= 0xD800 && n < 0xDC00) {
                c += this.string.charAt(this.i++);
            }
            this.parse('character', [c, this]);
        }
    };
    TexParser.prototype.Push = function (arg) {
        TreeHelper_js_1.TreeHelper.printMethod('Push (Old Parser Object)');
        this.stack.Push(arg);
    };
    TexParser.prototype.PushAll = function (args) {
        TreeHelper_js_1.TreeHelper.printMethod('PushAll (Old Parser Object)');
        for (var i = 0, m = args.length; i < m; i++) {
            this.stack.Push(args[i]);
        }
    };
    TexParser.prototype.mml = function () {
        TreeHelper_js_1.TreeHelper.printMethod('mml (Old Parser Object)');
        if (!this.stack.Top().hasType('mml')) {
            return null;
        }
        var node = this.stack.Top().data[0];
        node.setTeXclass(null);
        return node;
    };
    TexParser.prototype.mmlToken = function (token) {
        return token;
    };
    TexParser.prototype.convertDelimiter = function (c) {
        TreeHelper_js_1.TreeHelper.printMethod('convertDelimiter (Old Parser Object)');
        var symbol = this.lookup('delimiter', c);
        return symbol ? symbol.char : null;
    };
    TexParser.prototype.trimSpaces = function (text) {
        TreeHelper_js_1.TreeHelper.printMethod('trimSpaces (Old Parser Object)');
        if (typeof (text) != 'string') {
            return text;
        }
        var TEXT = text.replace(/^\s+|\s+$/g, '');
        if (TEXT.match(/\\$/) && text.match(/ $/))
            TEXT += ' ';
        return TEXT;
    };
    TexParser.prototype.nextIsSpace = function () {
        TreeHelper_js_1.TreeHelper.printMethod('nextIsSpace (Old Parser Object)');
        return this.string.charAt(this.i).match(/\s/);
    };
    TexParser.prototype.GetNext = function () {
        TreeHelper_js_1.TreeHelper.printMethod('GetNext (Old Parser Object)');
        while (this.nextIsSpace()) {
            this.i++;
        }
        return this.string.charAt(this.i);
    };
    TexParser.prototype.GetCS = function (name) {
        TreeHelper_js_1.TreeHelper.printMethod('GetCS (Old Parser Object)');
        var CS = this.string.slice(this.i).match(/^([a-z]+|.) ?/i);
        if (CS) {
            this.i += CS[1].length;
            return CS[1];
        }
        else {
            this.i++;
            return ' ';
        }
    };
    TexParser.prototype.GetArgument = function (name, noneOK) {
        TreeHelper_js_1.TreeHelper.printMethod('GetArgument (Old Parser Object)');
        switch (this.GetNext()) {
            case '':
                if (!noneOK) {
                    throw new TexError_js_1.default(['MissingArgFor', 'Missing argument for %1', name]);
                }
                return null;
            case '}':
                if (!noneOK) {
                    throw new TexError_js_1.default(['ExtraCloseMissingOpen',
                        'Extra close brace or missing open brace']);
                }
                return null;
            case '\\':
                this.i++;
                return '\\' + this.GetCS();
            case '{':
                var j = ++this.i, parens = 1;
                while (this.i < this.string.length) {
                    switch (this.string.charAt(this.i++)) {
                        case '\\':
                            this.i++;
                            break;
                        case '{':
                            parens++;
                            break;
                        case '}':
                            if (--parens == 0) {
                                return this.string.slice(j, this.i - 1);
                            }
                            break;
                    }
                }
                throw new TexError_js_1.default(['MissingCloseBrace', 'Missing close brace']);
        }
        return this.string.charAt(this.i++);
    };
    TexParser.prototype.GetBrackets = function (name, def) {
        TreeHelper_js_1.TreeHelper.printMethod('GetBrackets (Old Parser Object)');
        if (this.GetNext() != '[') {
            return def;
        }
        ;
        var j = ++this.i, parens = 0;
        while (this.i < this.string.length) {
            switch (this.string.charAt(this.i++)) {
                case '{':
                    parens++;
                    break;
                case '\\':
                    this.i++;
                    break;
                case '}':
                    if (parens-- <= 0) {
                        throw new TexError_js_1.default(['ExtraCloseLooking',
                            'Extra close brace while looking for %1', '\']\'']);
                    }
                    break;
                case ']':
                    if (parens == 0) {
                        return this.string.slice(j, this.i - 1);
                    }
                    break;
            }
        }
        throw new TexError_js_1.default(['MissingCloseBracket',
            'Could not find closing \']\' for argument to %1', name]);
    };
    TexParser.prototype.GetDelimiter = function (name, braceOK) {
        TreeHelper_js_1.TreeHelper.printMethod('GetDelimiter (Old Parser Object)');
        while (this.nextIsSpace()) {
            this.i++;
        }
        var c = this.string.charAt(this.i);
        this.i++;
        if (this.i <= this.string.length) {
            if (c == '\\') {
                c += this.GetCS(name);
            }
            else if (c === '{' && braceOK) {
                this.i--;
                c = this.GetArgument(name);
            }
            if (this.contains('delimiter', c)) {
                return this.convertDelimiter(c);
            }
        }
        throw new TexError_js_1.default(['MissingOrUnrecognizedDelim',
            'Missing or unrecognized delimiter for %1', name]);
    };
    TexParser.prototype.GetDimen = function (name) {
        TreeHelper_js_1.TreeHelper.printMethod('GetDimen (Old Parser Object)');
        if (this.nextIsSpace()) {
            this.i++;
        }
        if (this.string.charAt(this.i) == '{') {
            var dimen = this.GetArgument(name);
            if (dimen.match(/^\s*([-+]?([.,]\d+|\d+([.,]\d*)?))\s*(pt|em|ex|mu|px|mm|cm|in|pc)\s*$/)) {
                return dimen.replace(/ /g, '').replace(/,/, '.');
            }
        }
        else {
            var dimen = this.string.slice(this.i);
            var match = dimen.match(/^\s*(([-+]?([.,]\d+|\d+([.,]\d*)?))\s*(pt|em|ex|mu|px|mm|cm|in|pc)) ?/);
            if (match) {
                this.i += match[0].length;
                return match[1].replace(/ /g, '').replace(/,/, '.');
            }
        }
        throw new TexError_js_1.default(['MissingDimOrUnits',
            'Missing dimension or its units for %1', name]);
    };
    TexParser.prototype.GetUpTo = function (name, token) {
        TreeHelper_js_1.TreeHelper.printMethod('GetUpTo (Old Parser Object)');
        while (this.nextIsSpace()) {
            this.i++;
        }
        var j = this.i;
        var parens = 0;
        while (this.i < this.string.length) {
            var k = this.i;
            var c = this.string.charAt(this.i++);
            switch (c) {
                case '\\':
                    c += this.GetCS();
                    break;
                case '{':
                    parens++;
                    break;
                case '}':
                    if (parens == 0) {
                        throw new TexError_js_1.default(['ExtraCloseLooking',
                            'Extra close brace while looking for %1', token]);
                    }
                    parens--;
                    break;
            }
            if (parens == 0 && c == token) {
                return this.string.slice(j, k);
            }
        }
        throw new TexError_js_1.default(['TokenNotFoundForCommand',
            'Could not find %1 for %2', token, name]);
    };
    TexParser.prototype.ParseArg = function (name) {
        TreeHelper_js_1.TreeHelper.printMethod('ParseArg (Old Parser Object)');
        var object = new TexParser(this.GetArgument(name), this.stack.env);
        return object.mml();
    };
    TexParser.prototype.ParseUpTo = function (name, token) {
        TreeHelper_js_1.TreeHelper.printMethod('ParseUpTo (Old Parser Object)');
        return new TexParser(this.GetUpTo(name, token), this.stack.env).mml();
    };
    TexParser.prototype.InternalMath = function (text, level) {
        TreeHelper_js_1.TreeHelper.printMethod('InternalMath (Old Parser Object)');
        var def = (this.stack.env['font'] ? { mathvariant: this.stack.env['font'] } : {});
        var mml = [], i = 0, k = 0, c, node, match = '', braces = 0;
        if (text.match(/\\?[${}\\]|\\\(|\\(eq)?ref\s*\{/)) {
            while (i < text.length) {
                c = text.charAt(i++);
                if (c === '$') {
                    if (match === '$' && braces === 0) {
                        node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [(new TexParser(text.slice(k, i - 1), {})).mml()], {});
                        mml.push(node);
                        match = '';
                        k = i;
                    }
                    else if (match === '') {
                        if (k < i - 1)
                            mml.push(this.InternalText(text.slice(k, i - 1), def));
                        match = '$';
                        k = i;
                    }
                }
                else if (c === '{' && match !== '') {
                    braces++;
                }
                else if (c === '}') {
                    if (match === '}' && braces === 0) {
                        TreeHelper_js_1.TreeHelper.untested(12);
                        node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [(new TexParser(text.slice(k, i), {})).mml()], def);
                        mml.push(node);
                        match = '';
                        k = i;
                    }
                    else if (match !== '') {
                        if (braces)
                            braces--;
                    }
                }
                else if (c === '\\') {
                    if (match === '' && text.substr(i).match(/^(eq)?ref\s*\{/)) {
                        var len = RegExp['$&'].length;
                        if (k < i - 1)
                            mml.push(this.InternalText(text.slice(k, i - 1), def));
                        match = '}';
                        k = i - 1;
                        i += len;
                    }
                    else {
                        c = text.charAt(i++);
                        if (c === '(' && match === '') {
                            if (k < i - 2)
                                mml.push(this.InternalText(text.slice(k, i - 2), def));
                            match = ')';
                            k = i;
                        }
                        else if (c === ')' && match === ')' && braces === 0) {
                            TreeHelper_js_1.TreeHelper.untested(13);
                            node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [(new TexParser(text.slice(k, i - 2), {})).mml()], {});
                            mml.push(node);
                            match = '';
                            k = i;
                        }
                        else if (c.match(/[${}\\]/) && match === '') {
                            i--;
                            text = text.substr(0, i - 1) + text.substr(i);
                        }
                    }
                }
            }
            if (match !== '')
                throw new TexError_js_1.default(['MathNotTerminated', 'Math not terminated in text box']);
        }
        if (k < text.length)
            mml.push(this.InternalText(text.slice(k), def));
        if (level != null) {
            mml = [TreeHelper_js_1.TreeHelper.createNode('mstyle', mml, { displaystyle: false, scriptlevel: level })];
        }
        else if (mml.length > 1) {
            mml = [TreeHelper_js_1.TreeHelper.createNode('mrow', mml, {})];
        }
        return mml;
    };
    TexParser.prototype.InternalText = function (text, def) {
        TreeHelper_js_1.TreeHelper.printMethod('InternalText (Old Parser Object)');
        text = text.replace(/^\s+/, this.NBSP).replace(/\s+$/, this.NBSP);
        var textNode = TreeHelper_js_1.TreeHelper.createText(text);
        return TreeHelper_js_1.TreeHelper.createNode('mtext', [], def, textNode);
    };
    TexParser.prototype.GetDelimiterArg = function (name) {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-GetDelimiterArg (Old Parser Object)');
        var c = this.trimSpaces(this.GetArgument(name));
        if (c == '')
            return null;
        if (this.contains('delimiter', c)) {
            return c;
        }
        throw new TexError_js_1.default(['MissingOrUnrecognizedDelim', 'Missing or unrecognized delimiter for %1', name]);
    };
    TexParser.prototype.GetStar = function () {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-GetStar (Old Parser Object)');
        var star = (this.GetNext() === '*');
        if (star) {
            this.i++;
        }
        return star;
    };
    TexParser.prototype.GetRemap = function (char) {
        return this.remap.lookup(char);
    };
    return TexParser;
}());
exports.default = TexParser;
var SubHandler = (function () {
    function SubHandler(maps) {
        this._configuration = [];
        this._fallback = (function (x) { return null; });
        try {
            for (var maps_1 = __values(maps), maps_1_1 = maps_1.next(); !maps_1_1.done; maps_1_1 = maps_1.next()) {
                var name_3 = maps_1_1.value;
                this.add(name_3);
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (maps_1_1 && !maps_1_1.done && (_a = maps_1.return)) _a.call(maps_1);
            }
            finally { if (e_5) throw e_5.error; }
        }
        var e_5, _a;
    }
    SubHandler.prototype.fallback = function (method) {
        this._fallback = method;
    };
    SubHandler.prototype.add = function (name) {
        var map = MapHandler_js_1.default.getInstance().getMap(name);
        if (!map) {
            this.warn('Configuration ' + name + ' not found! Omitted.');
            return;
        }
        this._configuration.push(map);
    };
    SubHandler.prototype.parse = function (input) {
        try {
            for (var _a = __values(this._configuration), _b = _a.next(); !_b.done; _b = _a.next()) {
                var map = _b.value;
                var result = map.parse(input);
                if (result) {
                    return result;
                }
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_6) throw e_6.error; }
        }
        var _d = __read(input, 2), symbol = _d[0], env = _d[1];
        return this._fallback.bind(env)(env, symbol);
        var e_6, _c;
    };
    SubHandler.prototype.lookup = function (symbol) {
        var map = this.applicable(symbol);
        return map ? map.lookup(symbol) : null;
    };
    SubHandler.prototype.contains = function (symbol) {
        return this.applicable(symbol) ? true : false;
    };
    SubHandler.prototype.toString = function () {
        return this._configuration
            .map(function (x) { return x.name; })
            .join(', ');
    };
    SubHandler.prototype.applicable = function (symbol) {
        try {
            for (var _a = __values(this._configuration), _b = _a.next(); !_b.done; _b = _a.next()) {
                var map = _b.value;
                if (map.contains(symbol)) {
                    return map;
                }
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_7) throw e_7.error; }
        }
        return null;
        var e_7, _c;
    };
    SubHandler.prototype.warn = function (message) {
        console.log('TexParser Warning: ' + message);
    };
    return SubHandler;
}());
//# sourceMappingURL=TexParser.js.map