"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var TreeHelper_js_1 = require("./TreeHelper.js");
var TexParser_js_1 = require("./TexParser.js");
var TexError_js_1 = require("./TexError.js");
var mo_js_1 = require("../../core/MmlTree/MmlNodes/mo.js");
var NewTex;
(function (NewTex) {
    NewTex.display = false;
    function Compile(tex, display) {
        var script = {
            type: 'math/tex' + (display ? '; mode=display' : ''),
            innerText: tex,
            MathJax: {}
        };
        var node = Translate(script, [], {});
        node.setInheritedAttributes();
        node.setTeXclass();
        traverse(node);
        return node;
    }
    NewTex.Compile = Compile;
    var formatError = function (err, math, display, script) {
        var message = err.message.replace(/\n.*/, '');
        return TreeHelper_js_1.TreeHelper.createError(message);
    };
    function Translate(script, configurations, stackitem) {
        if (configurations === void 0) { configurations = []; }
        TreeHelper_js_1.TreeHelper.printMethod('Translate');
        TreeHelper_js_1.TreeHelper.printSimple(script.toString());
        var mml;
        var parser;
        var isError = false;
        var math = script.innerText;
        NewTex.display = script.type.replace(/\n/g, ' ').
            match(/(;|\s|\n)mode\s*=\s*display(;|\s|\n|$)/) != null;
        try {
            parser = new TexParser_js_1.default(math, null);
            mml = parser.mml();
            TreeHelper_js_1.TreeHelper.printSimple(mml.toString());
        }
        catch (err) {
            if (!(err instanceof TexError_js_1.default)) {
                throw err;
            }
            mml = formatError(err, math, NewTex.display, script);
            isError = true;
        }
        mml = TreeHelper_js_1.TreeHelper.cleanSubSup(mml);
        if (TreeHelper_js_1.TreeHelper.isType(mml, 'mtable') &&
            TreeHelper_js_1.TreeHelper.getAttribute(mml, 'displaystyle') === 'inherit') {
            TreeHelper_js_1.TreeHelper.untested('Tagged equations');
            TreeHelper_js_1.TreeHelper.setAttribute(mml, 'displaystyle', NewTex.display);
        }
        var mathNode = TreeHelper_js_1.TreeHelper.createNode('math', [mml], {});
        var root = TreeHelper_js_1.TreeHelper.getRoot(mathNode);
        if (NewTex.display) {
            TreeHelper_js_1.TreeHelper.setAttribute(root, 'display', 'block');
        }
        mathNode.setInheritedAttributes({}, false, 0, false);
        mathNode.setTeXclass(null);
        if (!parser) {
            return mathNode;
        }
        try {
            for (var _a = __values(parser.secondPass), _b = _a.next(); !_b.done; _b = _a.next()) {
                var mo = _b.value;
                var forms = mo.getForms();
                var symbol = void 0;
                var range = null;
                try {
                    for (var forms_1 = __values(forms), forms_1_1 = forms_1.next(); !forms_1_1.done; forms_1_1 = forms_1.next()) {
                        var form = forms_1_1.value;
                        symbol = mo_js_1.MmlMo.OPTABLE[form][mo.getText()];
                        if (symbol) {
                            break;
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (forms_1_1 && !forms_1_1.done && (_c = forms_1.return)) _c.call(forms_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                if (symbol && symbol[3] && symbol[3]['stretchy']) {
                    TreeHelper_js_1.TreeHelper.setAttribute(mo, 'stretchy', false);
                }
                if (!TreeHelper_js_1.TreeHelper.getTexClass(mo) && (!symbol || !symbol[2])) {
                    var parent_1 = mo.parent;
                    var texAtom = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [mo], {});
                    texAtom.parent = parent_1;
                    parent_1.childNodes = [texAtom];
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_d = _a.return)) _d.call(_a);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return mathNode;
        var e_2, _d, e_1, _c;
    }
    NewTex.Translate = Translate;
    function traverse(mml) {
        var attribs = mml.attributes;
        var keys = Object.keys(attribs.attributes);
        for (var i = 0, key = void 0; key = keys[i]; i++) {
            if (attribs.attributes[key] === mml.attributes.getInherited(key)) {
                delete attribs.attributes[key];
            }
        }
        if (!mml.isToken) {
            var children = TreeHelper_js_1.TreeHelper.getChildren(mml);
            for (var i = 0, m = children.length; i < m; i++) {
                traverse(children[i]);
            }
        }
    }
    ;
})(NewTex = exports.NewTex || (exports.NewTex = {}));
//# sourceMappingURL=Translate.js.map