import { AbstractWrapper } from '../../core/Tree/Wrapper.js';
import { PropertyList } from '../../core/Tree/Node.js';
import { MmlNode, TextNode } from '../../core/MmlTree/MmlNode.js';
import { Property } from '../../core/Tree/Node.js';
import { OptionList } from '../../util/Options.js';
import { Styles } from '../../util/Styles.js';
import { DOMAdaptor } from '../../core/DOMAdaptor.js';
import { CHTML } from '../chtml.js';
import { CHTMLWrapperFactory } from './WrapperFactory.js';
import { CHTMLmo } from './Wrappers/mo.js';
import { BBox } from './BBox.js';
import { FontData, DelimiterData, DIRECTION } from './FontData.js';
import { StyleList } from './CssStyles.js';
export declare type StringMap = {
    [key: string]: string;
};
export declare const FONTSIZE: StringMap;
export declare const SPACE: StringMap;
export declare class CHTMLWrapper<N, T, D> extends AbstractWrapper<MmlNode, CHTMLWrapper<N, T, D>> {
    static kind: string;
    static autoStyle: boolean;
    static styles: StyleList;
    static removeStyles: [string];
    static skipAttributes: {
        [name: string]: boolean;
    };
    static BOLDVARIANTS: {
        [name: string]: StringMap;
    };
    static ITALICVARIANTS: {
        [name: string]: StringMap;
    };
    protected factory: CHTMLWrapperFactory<N, T, D>;
    parent: CHTMLWrapper<N, T, D>;
    childNodes: CHTMLWrapper<N, T, D>[];
    chtml: N;
    protected removedStyles: StringMap;
    protected styles: Styles;
    variant: string;
    bbox: BBox;
    protected bboxComputed: boolean;
    stretch: DelimiterData;
    font: FontData;
    readonly CHTML: CHTML<N, T, D>;
    readonly adaptor: DOMAdaptor<N, T, D>;
    readonly metrics: {
        em: number;
        ex: number;
        containerWidth: number;
        lineWidth: number;
        scale: number;
    };
    constructor(factory: CHTMLWrapperFactory<N, T, D>, node: MmlNode, parent?: CHTMLWrapper<N, T, D>);
    wrap(node: MmlNode, parent?: CHTMLWrapper<N, T, D>): CHTMLWrapper<N, T, D>;
    toCHTML(parent: N): void;
    getBBox(save?: boolean): BBox;
    protected computeBBox(bbox: BBox): void;
    protected getStyles(): void;
    protected getVariant(): void;
    protected explicitVariant(fontFamily: string, fontWeight: string, fontStyle: string): string;
    protected getScale(): void;
    protected getSpace(): void;
    protected standardCHTMLnode(parent: N): N;
    protected createCHTMLnode(parent: N): N;
    protected handleStyles(): void;
    protected handleVariant(): void;
    protected handleScale(): void;
    protected setScale(chtml: N, rscale: number): N;
    protected handleSpace(): void;
    protected handleColor(): void;
    protected handleAttributes(): void;
    core(): CHTMLWrapper<N, T, D>;
    coreMO(): CHTMLmo<N, T, D>;
    getText(): string;
    canStretch(direction: DIRECTION): boolean;
    drawBBox(): void;
    protected percent(m: number): string;
    protected em(m: number): string;
    protected px(m: number, M?: number): string;
    protected length2em(length: Property, size?: number, scale?: number): number;
    protected unicodeChars(text: string): number[];
    protected char(n: number, escape?: boolean): string;
    html(type: string, def?: OptionList, content?: N[]): N;
    text(text: string): T;
    mmlText(text: string): TextNode;
    mmlNode(kind: string, properties?: PropertyList, children?: MmlNode[]): MmlNode;
    protected createMo(text: string): CHTMLmo<N, T, D>;
}
export declare type CHTMLWrapperClass = typeof CHTMLWrapper;
